from django import forms
from django.test.testcases import SimpleTestCase

from form_tags.templatetags.forms import field_tag
from tests.mixins import DummyFormMixin


class FormFieldTestCase(DummyFormMixin, SimpleTestCase):
    maxDiff = None

    def test_field_attributes(self):
        html_output = field_tag(self.form['char_field'], **{
            'class': 'test-class awesome',
            'any': 'attr'
        })
        self.assertHTMLEqual(html_output, '''
            <input class="input-field test-class awesome" any="attr" id="id_char_field" name="char_field" type="text" value="abc" />
        ''')  # noqa

        html_output = field_tag(self.form['text_field'], **{
            'class': 'test-class awesome',
            'any': 'attr'
        })
        self.assertHTMLEqual(html_output, '''
            <textarea class="test-class awesome" any="attr" cols="40" id="id_text_field" name="text_field" rows="10">abc</textarea>
        ''')  # noqa

        html_output = field_tag(self.form['char_field'])
        self.assertHTMLEqual(html_output, '''
            <input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" />
        ''')  # noqa

    def test_field_text_input(self):
        html_output = field_tag(self.form['char_field'])
        self.assertHTMLEqual(html_output, '''
            <input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" />
        ''')  # noqa

        html_output = field_tag(self.form['text_field'])
        self.assertHTMLEqual(html_output, '''
            <textarea class="" cols="40" id="id_text_field" name="text_field" rows="10">abc</textarea>
        ''')  # noqa

        self.form['char_field'].field.widget.attrs.update({'class': 'test'})
        self.form['text_field'].field.widget.attrs.update({'class': 'test'})

        html_output = field_tag(self.form['char_field'])
        self.assertHTMLEqual(html_output, '''
            <input class="input-field test" id="id_char_field" name="char_field" type="text" value="abc" />
        ''')  # noqa

        html_output = field_tag(self.form['text_field'])
        self.assertHTMLEqual(html_output, '''
            <textarea class="test" cols="40" id="id_text_field" name="text_field" rows="10">abc</textarea>
        ''')  # noqa

    def test_field_boolean_attrs(self):
        html_output = field_tag(self.form['char_field'], **{
            'autofocus': True,
            'checked': 1,
            'disabled': 'yes',
            'formnovalidate': 'formnovalidate',
            'multiple': '1',
            'readonly': '1',
            'required': '1',
        })
        self.assertHTMLEqual(html_output, '''
            <input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" autofocus="autofocus" checked="checked" disabled="disabled" formnovalidate="formnovalidate" multiple="multiple" readonly="readonly" required="required" />
        ''')  # noqa

        html_output = field_tag(self.form['char_field'], **{
            'autofocus': False,
            'checked': 0,
            'disabled': '',
        })
        self.assertHTMLEqual(html_output, '''
            <input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" />
        ''')  # noqa

        with self.settings(FIELD_BOOLEAN_ATTRS=['test']):
            html_output = field_tag(self.form['char_field'], **{
                'autofocus': '1',
                'checked': '2',
                'disabled': '3',
                'formnovalidate': '4',
                'multiple': '5',
                'readonly': '6',
                'required': '7',
                'test': '8'
            })
            self.assertHTMLEqual(html_output, '''
                <input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" autofocus="1" checked="2" disabled="3" formnovalidate="4" multiple="5" readonly="6" required="7" test="test" />
            ''')  # noqa

    def test_field_suppress_errors(self):
        self.form.add_error('char_field', forms.ValidationError('Err'))

        html_output = field_tag(self.form['char_field'], suppress_errors=True)
        self.assertHTMLEqual(html_output, '''
            <input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" />
        ''')  # noqa

        html_output = field_tag(self.form['char_field'], suppress_errors=False)
        self.assertHTMLEqual(html_output, '''
            <input class="input-field error" id="id_char_field" name="char_field" type="text" value="abc" />
        ''')  # noqa

        html_output = field_tag(self.form['char_field'])
        self.assertHTMLEqual(html_output, '''
            <input class="input-field error" id="id_char_field" name="char_field" type="text" value="abc" />
        ''')  # noqa



import django
from django import forms
from django.forms.formsets import formset_factory
from django.test.testcases import SimpleTestCase

from form_tags.templatetags.forms import hidden_fields, non_field_errors, \
    non_form_errors, _add_error_class, _help_text, _get_field_kwargs, \
    _unit_text
from tests.mixins import DummyFormMixin, DummyFormSet, DummyForm


class UtilsTestCase(DummyFormMixin, SimpleTestCase):
    maxDiff = None

    def test_add_error_class(self):
        assert _add_error_class(None, self.form['char_field'], False) == ''
        assert _add_error_class('', self.form['char_field'], False) == ''
        assert _add_error_class('test', self.form['char_field'], False) == 'test'  # noqa
        assert _add_error_class('test multiple', self.form['char_field'], False) == 'test multiple'  # noqa
        assert _add_error_class(' test multiple  ', self.form['char_field'], False) == 'test multiple'  # noqa

        self.form.add_error('char_field', forms.ValidationError('Err'))

        assert _add_error_class(None, self.form['char_field'], False) == 'error'  # noqa
        assert _add_error_class('', self.form['char_field'], False) == 'error'
        assert _add_error_class('test', self.form['char_field'], False) == 'test error'  # noqa
        assert _add_error_class('test multiple', self.form['char_field'], False) == 'test multiple error'  # noqa
        assert _add_error_class(' test multiple  ', self.form['char_field'], False) == 'test multiple error'  # noqa

        assert _add_error_class(None, self.form['char_field'], True) == ''
        assert _add_error_class('', self.form['char_field'], True) == ''
        assert _add_error_class('test', self.form['char_field'], True) == 'test'  # noqa
        assert _add_error_class('test multiple', self.form['char_field'], True) == 'test multiple'  # noqa
        assert _add_error_class(' test multiple  ', self.form['char_field'], True) == 'test multiple'  # noqa

    def test_help_text(self):
        assert _help_text(None) == ''
        assert _help_text('') == ''
        assert _help_text('Help me!') == '<div class="helptext">Help me!</div>'

    def test_get_field_kwargs(self):
        kwargs = {
            'some_arg': 'test',
            'field0_class': '',
            'field1_class': 'test',
            'field1_data_some': '',
        }
        assert _get_field_kwargs(0, **kwargs) == {
            'class': ''
        }
        assert _get_field_kwargs(1, **kwargs) == {
            'class': 'test',
            'data_some': '',
        }
        assert _get_field_kwargs(3, **kwargs) == {}
        assert _get_field_kwargs(0) == {}

    def test_unit_text(self):
        assert _unit_text(None, None) == {}
        assert _unit_text('', '') == {}
        assert _unit_text('$', '') == {
            'fieldwrapper_class': 'fieldwrapper--unit',
            'before_field': '<i class="input-unit input-unit--left">$</i>\n'
        }
        assert _unit_text('', 'KM') == {
            'fieldwrapper_class': 'fieldwrapper--unit',
            'after_field': '<i class="input-unit input-unit--right">KM</i>\n'
        }
        assert _unit_text('&', 'KM') == {
            'fieldwrapper_class': 'fieldwrapper--unit',
            'before_field':
                '<i class="input-unit input-unit--left">&amp;</i>\n',
            'after_field': '<i class="input-unit input-unit--right">KM</i>\n'
        }
        assert _unit_text('&', 'KM', 'fieldwrapper--icon') == {}
        assert _unit_text('&', 'KM', '  other-class  ') == {
            'fieldwrapper_class': 'fieldwrapper--unit other-class',
            'before_field':
                '<i class="input-unit input-unit--left">&amp;</i>\n',
            'after_field': '<i class="input-unit input-unit--right">KM</i>\n'
        }

    def test_hidden_fields(self):
        html_output = hidden_fields(self.form)
        self.assertHTMLEqual(html_output, '''
            <input id="id_hidden_field" name="hidden_field" type="hidden" />
            <input id="id_another_hidden_field" name="another_hidden_field" type="hidden" />
        ''')  # noqa

    def test_non_field_errors(self):
        html_output = non_field_errors(self.form)
        assert html_output == ''

        self.form.add_error(None, forms.ValidationError('test'))
        html_output = non_field_errors(self.form)

        if django.VERSION >= (1, 8, 0):
            self.assertHTMLEqual(html_output, '''
                <div class="inline-validation inline-validation--error">
                   <ul class="errorlist nonfield"><li>test</li></ul>
                </div>
            ''')
        else:
            self.assertHTMLEqual(html_output, '''
                <div class="inline-validation inline-validation--error">
                   <ul class="errorlist"><li>test</li></ul>
                </div>
            ''')

    def test_non_form_errors(self):
        formset = formset_factory(DummyForm, DummyFormSet)
        dummy_formset = formset()
        html_output = non_form_errors(dummy_formset)
        assert html_output == ''

        dummy_formset.add_error()
        html_output = non_form_errors(dummy_formset)
        self.assertHTMLEqual(html_output, '''
            <div class="inline-validation inline-validation--error">
               <ul class="errorlist"><li>Non form error 2</li></ul>
            </div>
        ''')



from django import forms
from svg_templatetag.templatetags.svg import get_svg


class DummyFormSet(forms.BaseFormSet):
    def add_error(self):
        self._non_form_errors = self.error_class(
            forms.ValidationError("Non form error 2").error_list)


class DummyForm(forms.Form):
    use_required_attribute = False
    char_field = forms.CharField()
    int_field = forms.IntegerField()
    date_field = forms.DateField()
    checkbox_field = forms.BooleanField()
    checkbox_help_field = forms.BooleanField(help_text='Help Me!')
    text_field = forms.CharField(widget=forms.Textarea)
    pass_field = forms.CharField(widget=forms.PasswordInput)
    choice_field = forms.ChoiceField(choices=[('1', 'a'), ('2', 'b')])
    radio_field = forms.ChoiceField(
        widget=forms.RadioSelect, choices=[('1', 'a'), ('2', 'b')])
    help_field = forms.CharField(help_text='Help Text')
    hidden_field = forms.CharField(widget=forms.HiddenInput)
    another_hidden_field = forms.Field(widget=forms.HiddenInput)


class DummyFormMixin(object):
    def setUp(self):
        self.form = DummyForm({
            'char_field': 'abc',
            'int_field': 1,
            'date_field': '1940-10-9',
            'text_field': 'abc',
            'checkbox_field': True,
            'checkbox_help_field': True,
            'pass_field': 'secret',
            'choice_field': '1',
            'radio_field': '1',
            'help_field': 'abc',
        })


class SvgIconsMixin(object):
    @classmethod
    def setUpClass(cls):
        super(SvgIconsMixin, cls).setUpClass()
        cls.eye_svg = get_svg(
            'form_tags/icon_eye.svg',
            **{'class': 'icn icn--ic-24 icn--ic-remove-red-eye'})

        cls.edit_svg = get_svg(
            'form_tags/icon_edit.svg',
            **{'class': 'icn icn--ic-24 icn--ic-edit'})

        cls.date_range_svg = get_svg(
            'form_tags/icon_date_range.svg',
            **{'class': 'icn icn--ic-24 icn--ic-date-range'})



import re

from django import template
from django.forms import widgets
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from svg_templatetag.templatetags.svg import get_svg

from ..conf import settings


register = template.Library()

TEXT_INPUT_WIDGETS = (
    widgets.TextInput, widgets.NumberInput, widgets.EmailInput,
    widgets.URLInput, widgets.PasswordInput
)

FIELD_HOLDER = '''
    <div class="fieldholder {fieldholder_class}">
        {label}
        {field}
    </div>'''

FIELD_HOLDER_INLINE = '''
    <td class="table__cell table__cell--inline-edit {fieldholder_class}">
        {field}
    </td>'''

FIELD_WRAPPER = '''
    <div class="fieldwrapper fieldwrapper--{field_type} fieldwrapper--{widget_type} {fieldwrapper_class}">
        {before_field}
        {field}
        {after_field}
        {help_text}
        {errors}
    </div>'''  # noqa

FIELD_WRAPPER_SELECT = FIELD_WRAPPER.replace(
        '{field}',
        '<div class="select">{field}</div>'
    )

FIELD_WRAPPER_CHECKBOX = FIELD_WRAPPER.replace(
        '{field}',
        '''<label class="control checkbox">
            {field}
            <span class="control-indicator"></span>
            <span class="control-label">{control_label}</span>
         </label>'''
    )

COMBINED_FIELD_WRAPPER = '''
    <div class="fieldwrapper {fieldwrapper_class}">
        <div class="fieldwrapper-combined">
            {fields}
        </div>
        {help_text}
        {errors}
    </div>'''

COMBINED_FIELD = '''
    <div class="fieldwrapper-combined__field fieldwrapper--{field_type} fieldwrapper--{widget_type} {fieldwrapper_class}">
       {before_field}
       {field}
       {after_field}
    </div>'''  # noqa

INLINE_VALIDATION = '''
    <div class="inline-validation inline-validation--error">
        {errors}
    </div>'''

BUTTON_PASSWORD = format_html(
    '''<button class="input-icon input-icon__password" type="button" tabindex="-1">
        {icon}
    </button>''',  # noqa
    icon=get_svg(
        'form_tags/icon_eye.svg',
        **{'class': 'icn icn--ic-24 icn--ic-remove-red-eye'}
    ))

LABEL_INLINE_EDIT = format_html(
    '''<label class="label--inline-edit" for="{{for}}" title="{{title}}">
        {icon}
    </label>''',
    icon=get_svg(
        'form_tags/icon_edit.svg',
        **{'class': 'icn icn--ic-24 icn--ic-edit'}
    ))

LABEL_INLINE_EDIT_DATE = format_html(
    '''<label class="label--inline-edit" for="{{for}}" title="{{title}}">
        {icon}
    </label>''',
    icon=get_svg(
        'form_tags/icon_date_range.svg',
        **{'class': 'icn icn--ic-24 icn--ic-date-range'}
    ))


def _add_error_class(classes, field, suppress_errors):
    classes = (classes or '').strip()
    if not suppress_errors and len(field.errors):
        return (classes + ' error').strip()
    return classes


def _help_text(help_text):
    if help_text:
        return format_html(
            '<div class="helptext">{}</div>', help_text)
    return help_text or ''


def _get_field_kwargs(k, **kwargs):
    regex = re.compile(r'field{}_'.format(k))
    return {regex.sub('', key): value
            for key, value in kwargs.items() if regex.match(key)}


def _unit_text(left, right, fieldwrapper_class=''):
    format_kwargs = {}
    if 'fieldwrapper--icon' not in fieldwrapper_class:
        fieldwrapper_class = fieldwrapper_class.strip()
        # left unit icon
        if left:
            format_kwargs['fieldwrapper_class'] = ('fieldwrapper--unit ' + fieldwrapper_class).strip()  # noqa
            format_kwargs['before_field'] = format_html(
                '<i class="input-unit input-unit--left">{}</i>\n', left)
        # right unit icon
        if right:
            if 'fieldwrapper--unit' not in fieldwrapper_class:
                format_kwargs['fieldwrapper_class'] = ('fieldwrapper--unit ' + fieldwrapper_class).strip()  # noqa
            format_kwargs['after_field'] = format_html(
                '<i class="input-unit input-unit--right">{}</i>\n', right)
    return format_kwargs


@register.simple_tag
def hidden_fields(form):
    return mark_safe('\n'.join(
        [field.as_widget() for field in form.hidden_fields()]
    ))


@register.simple_tag
def non_field_errors(form):
    if len(form.non_field_errors()):
        return format_html(
            INLINE_VALIDATION, errors=mark_safe(form.non_field_errors()))
    return ''


@register.simple_tag
def non_form_errors(formset):
    if len(formset.non_form_errors()):
        return format_html(
            INLINE_VALIDATION, errors=mark_safe(formset.non_form_errors()))
    return ''


@register.simple_tag(name='field')
def field_tag(field, suppress_errors=False, **kwargs):
    if isinstance(field.field.widget, TEXT_INPUT_WIDGETS):
        kwargs.update({
            'class': ('input-field ' + kwargs.get('class', '').strip()).strip()
        })
    classes = _add_error_class(kwargs.get('class', ''), field, suppress_errors)

    attrs = field.field.widget.attrs.copy()
    kwargs['class'] = (attrs.get('class', '') + ' ' + classes).strip()

    for attr in settings.FIELD_BOOLEAN_ATTRS:
        if attr in kwargs:
            if not kwargs[attr]:
                kwargs.pop(attr)
            else:
                kwargs[attr] = attr

    attrs.update({k.replace('_', '-'): v for k, v, in kwargs.items()})
    return field.as_widget(attrs=attrs)


@register.simple_tag
def fieldwrapper(field, fieldwrapper_class='', before_field='', after_field='',
                 control_label=None, unit_text_left='', unit_text_right='',
                 **kwargs):
    suppress_errors = kwargs.get('suppress_errors', False)
    classes = _add_error_class(fieldwrapper_class, field, suppress_errors)

    format_kwargs = {
        'fieldwrapper_class': classes,
        'field_type': field.field.__class__.__name__.lower(),
        'widget_type': field.field.widget.__class__.__name__.lower(),
        'before_field': before_field,
        'after_field': after_field,
        'errors': mark_safe(field.errors) if not suppress_errors else '',
        'help_text': _help_text(field.help_text)
    }

    if isinstance(field.field.widget, widgets.Select) and \
            not isinstance(field.field.widget, widgets.RadioSelect):
        field_wrapper = FIELD_WRAPPER_SELECT
    elif isinstance(field.field.widget, widgets.CheckboxInput):
        field_wrapper = FIELD_WRAPPER_CHECKBOX
        if control_label is not None:
            format_kwargs['control_label'] = control_label
        elif field.help_text:
            format_kwargs['control_label'] = field.help_text
            format_kwargs['help_text'] = ''
        else:
            format_kwargs['control_label'] = field.label
    elif isinstance(field.field.widget, widgets.PasswordInput):
        field_wrapper = FIELD_WRAPPER
        format_kwargs['fieldwrapper_class'] = ('fieldwrapper--icon ' + format_kwargs.get('fieldwrapper_class', '')).strip()  # noqa
        format_kwargs['after_field'] = BUTTON_PASSWORD
    elif isinstance(field.field.widget, widgets.DateTimeBaseInput):
        field_wrapper = FIELD_WRAPPER
        format_kwargs['fieldwrapper_class'] = ('fieldwrapper--icon ' + format_kwargs.get('fieldwrapper_class', '')).strip()  # noqa
        format_kwargs['after_field'] = format_html(
            LABEL_INLINE_EDIT_DATE,
            **{
                'for': field.id_for_label,
                'title': _('Edit {}').format(field.label),
            }
        )
    else:
        field_wrapper = FIELD_WRAPPER

    if isinstance(field.field.widget, TEXT_INPUT_WIDGETS):
        format_kwargs.update(_unit_text(unit_text_left, unit_text_right,
                                        format_kwargs['fieldwrapper_class']))

    if 'input-unit--right' in format_kwargs['after_field']:
        kwargs['class'] = ('input-field--unit-right ' + kwargs.get('class', '')).strip()  # noqa
    if 'input-unit--left' in format_kwargs['before_field']:
        kwargs['class'] = ('input-field--unit-left ' + kwargs.get('class', '')).strip()  # noqa

    format_kwargs['field'] = field_tag(field, **kwargs)

    return format_html(field_wrapper, **format_kwargs)


@register.simple_tag(takes_context=True)
def fieldholder(context, field, fieldholder_class='', horizontal=None,
                label_tag=None, label=None, **kwargs):

    if horizontal is None and context.get('horizontal') or horizontal:
        fieldholder_class = ('fieldholder--horizontal ' + fieldholder_class).strip()  # noqa

    suppress_errors = kwargs['suppress_errors'] = \
        kwargs.get('suppress_errors', context.get('suppress_errors', False))
    classes = _add_error_class(fieldholder_class, field, suppress_errors)

    if label_tag is None and label is not None:
        label_tag = format_html(
            '<label for="{id_for_label}" title="{label}">{label}</label>',
            id_for_label=field.id_for_label, label=label)

    return format_html(
        FIELD_HOLDER,
        field=fieldwrapper(field, **kwargs),
        label=label_tag if label_tag is not None else field.label_tag(
            attrs={'title': field.label}),
        fieldholder_class=classes
    )


@register.simple_tag(takes_context=True)
def fieldholder_inline(context, field, fieldholder_class='', label_tag=None,
                       **kwargs):
    kwargs.pop('unit_text_left', None)
    kwargs.pop('unit_text_right', None)

    if isinstance(field.field.widget, widgets.ChoiceWidget):
        kwargs.update({
            'fieldwrapper_class': ('select--inline-edit ' + kwargs.get('fieldwrapper_class', '')).strip(),  # noqa
        })
    elif isinstance(field.field.widget, TEXT_INPUT_WIDGETS):
        kwargs.update({
            'fieldwrapper_class': ('fieldwrapper--inline-edit ' + kwargs.get('fieldwrapper_class', '')).strip(),  # noqa
            'class': ('input-field--inline-edit ' + kwargs.get('class', '')).strip(),  # noqa
            'after_field': format_html(
                LABEL_INLINE_EDIT,
                **{
                    'for': field.id_for_label,
                    'title': _('Edit {}').format(field.label),
                }
            ) if label_tag is None else label_tag,
            'placeholder': kwargs.get('placeholder', field.label)
        })

    suppress_errors = kwargs['suppress_errors'] = \
        kwargs.get('suppress_errors', context.get('suppress_errors', False))
    classes = _add_error_class(fieldholder_class, field, suppress_errors)

    return format_html(
        FIELD_HOLDER_INLINE,
        field=fieldwrapper(field, **kwargs),
        fieldholder_class=classes
    )


def field_tags(*fields, **kwargs):
    suppress_errors = kwargs.pop('suppress_errors', False)

    def get_format_kwargs():
        for k, field in enumerate(fields):
            field_kwargs = _get_field_kwargs(k, **kwargs)
            before_field = field_kwargs.pop('before_field', '')
            after_field = field_kwargs.pop('after_field', '')
            unit_text_left = field_kwargs.pop('unit_text_left', '')
            unit_text_right = field_kwargs.pop('unit_text_right', '')

            format_kwargs = {
                'fieldwrapper_class': _add_error_class('', field, suppress_errors),  # noqa
                'field_type': field.field.__class__.__name__.lower(),
                'widget_type': field.field.widget.__class__.__name__.lower(),
                'before_field': before_field,
                'after_field': after_field,
            }

            if isinstance(field.field.widget, widgets.ChoiceWidget):
                format_kwargs['fieldwrapper_class'] = ('select ' + format_kwargs.get('fieldwrapper_class', '')).strip()  # noqa
            elif isinstance(field.field.widget, widgets.PasswordInput):
                format_kwargs['fieldwrapper_class'] = ('fieldwrapper--icon ' + format_kwargs.get('fieldwrapper_class', '')).strip()  # noqa
                format_kwargs['after_field'] = BUTTON_PASSWORD
            elif isinstance(field.field.widget, widgets.DateTimeBaseInput):
                format_kwargs['fieldwrapper_class'] = ('fieldwrapper--icon ' + format_kwargs.get('fieldwrapper_class', '')).strip()  # noqa
                format_kwargs['after_field'] = format_html(
                    LABEL_INLINE_EDIT_DATE,
                    **{
                        'for': field.id_for_label,
                        'title': _('Edit {}').format(field.label),
                    }
                )

            if isinstance(field.field.widget, TEXT_INPUT_WIDGETS) and \
                    'placeholder' not in field.field.widget.attrs:
                field_kwargs['placeholder'] = \
                    field_kwargs.get('placeholder', field.label)

                format_kwargs.update(_unit_text(
                    unit_text_left, unit_text_right,
                    format_kwargs['fieldwrapper_class']))

            if 'input-unit--right' in format_kwargs['after_field']:
                field_kwargs['class'] = ('input-field--unit-right ' + field_kwargs.get('class', '')).strip()  # noqa
            if 'input-unit--left' in format_kwargs['before_field']:
                field_kwargs['class'] = ('input-field--unit-left ' + field_kwargs.get('class', '')).strip()  # noqa

            format_kwargs['field'] = field_tag(
                field, suppress_errors=suppress_errors, **field_kwargs)

            yield format_kwargs

    return mark_safe('\n'.join(format_html(COMBINED_FIELD, **fkwargs)
                               for fkwargs in get_format_kwargs()))


@register.simple_tag
def fieldwrapper_combined(*fields, **kwargs):
    suppress_errors = kwargs.get('suppress_errors', False)

    format_kwargs = {
        'fieldwrapper_class': kwargs.pop('fieldwrapper_class', '').strip(),
        'fields': field_tags(*fields, **kwargs),
        'errors': (mark_safe('\n'.join([str(x.errors) for x in fields]))
                   if not suppress_errors else ''),
        'help_text':
            mark_safe('\n'.join([_help_text(x.help_text) for x in fields]))
    }

    return format_html(COMBINED_FIELD_WRAPPER, **format_kwargs)


@register.simple_tag(takes_context=True)
def fieldholder_combined(context, *fields, **kwargs):
    fieldholder_class = kwargs.pop('fieldwrapper_class', '')
    horizontal = kwargs.pop('horizontal', None)
    label_tag = kwargs.pop('label_tag', None)
    label = kwargs.pop('label', None)

    if horizontal is None and context.get('horizontal') or horizontal:
        fieldholder_class = ('fieldholder--horizontal ' + fieldholder_class).strip()  # noqa

    kwargs['suppress_errors'] = \
        kwargs.get('suppress_errors', context.get('suppress_errors', False))

    if label_tag is None and label is not None:
        label_tag = format_html(
            '<label for="{id_for_label}" title="{label}">{label}</label>',
            id_for_label=fields[0].id_for_label, label=label)

    return format_html(
        FIELD_HOLDER,
        field=fieldwrapper_combined(*fields, **kwargs),
        label=label_tag if label_tag is not None else fields[0].label_tag(
            attrs={'title': fields[0].label}),
        fieldholder_class=fieldholder_class
    )



from django.conf import settings


class LazySetting(object):
    def __init__(self, name, default=None):
        self.name = name
        self.default = default

    def __get__(self, obj, cls):
        if obj is None:
            return self
        return getattr(obj._settings, self.name, self.default)


class LazySettings(object):
    def __init__(self, settings):
        self._settings = settings

    FIELD_BOOLEAN_ATTRS = LazySetting(
        name='FIELD_BOOLEAN_ATTRS',
        default=[
            'autofocus',
            'checked',
            'disabled',
            'formnovalidate',
            'multiple',
            'readonly',
            'required',
        ],
    )


settings = LazySettings(settings)



from django.test.testcases import SimpleTestCase
from django.utils.translation import activate

from form_tags.templatetags.forms import fieldwrapper_combined
from tests.mixins import DummyFormMixin, SvgIconsMixin


class FormFieldwrapperCombinedTestCase(
        DummyFormMixin, SvgIconsMixin, SimpleTestCase):
    maxDiff = None

    # TODO add fieldwrapper

    # TODO add fieldholder_inline

    def test_fieldwrapper_date_label_title_translation(self):
        activate('en')
        html_output = fieldwrapper_combined(
            self.form['date_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--datefield fieldwrapper--dateinput fieldwrapper--icon">
                        <input class="input-field" id="id_date_field" name="date_field" type="text" value="1940-10-9" placeholder="Date field" />
                        <label class="label--inline-edit" for="id_date_field" title="Edit Date field">{}</label>
                    </div>
                </div>
            </div>
        '''.format(self.date_range_svg))  # noqa

        activate('nl')
        html_output = fieldwrapper_combined(
            self.form['date_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--datefield fieldwrapper--dateinput fieldwrapper--icon">
                        <input class="input-field" id="id_date_field" name="date_field" type="text" value="1940-10-9" placeholder="Date field" />
                        <label class="label--inline-edit" for="id_date_field" title="Date field wijzigen">{}</label>
                    </div>
                </div>
            </div>
        '''.format(self.date_range_svg))  # noqa



from django import forms
from django.test.testcases import SimpleTestCase
from django.utils.safestring import mark_safe

from form_tags.templatetags.forms import fieldwrapper_combined, fieldwrapper
from tests.mixins import DummyFormMixin, SvgIconsMixin


class FormFieldwrapperTestCase(DummyFormMixin, SvgIconsMixin, SimpleTestCase):
    maxDiff = None

    def test_fieldwrapper(self):
        html_output = fieldwrapper(
            self.form['char_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--charfield fieldwrapper--textinput ">
                <input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" />
            </div>
        ''')  # noqa

    def test_fieldwrapper_class(self):
        html_output = fieldwrapper(
            self.form['char_field'],
            fieldwrapper_class='  test class  ',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--charfield fieldwrapper--textinput test class">
                <input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" />
            </div>
        ''')  # noqa

    def test_fieldwrapper_errors(self):
        self.form.add_error('char_field', forms.ValidationError('Err'))

        html_output = fieldwrapper(
            self.form['char_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--charfield fieldwrapper--textinput error">
                <input class="input-field error" id="id_char_field" name="char_field" type="text" value="abc" />
                <ul class="errorlist"><li>Err</li></ul>
            </div>
        ''')  # noqa

        html_output = fieldwrapper(
            self.form['char_field'],
            suppress_errors=True
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--charfield fieldwrapper--textinput ">
                <input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" />
            </div>
        ''')  # noqa

    def test_fieldwrapper_help_text(self):
        html_output = fieldwrapper(
            self.form['help_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--charfield fieldwrapper--textinput ">
                <input class="input-field" id="id_help_field" name="help_field" type="text" value="abc" />
                <div class="helptext">Help Text</div>
            </div>
        ''')  # noqa

        self.form.add_error('help_field', forms.ValidationError('Err'))

        html_output = fieldwrapper(
            self.form['help_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--charfield fieldwrapper--textinput error">
                <input class="input-field error" id="id_help_field" name="help_field" type="text" value="abc" />
                <div class="helptext">Help Text</div>
                <ul class="errorlist"><li>Err</li></ul>
            </div>
        ''')  # noqa

    def test_fieldwrapper_select(self):
        html_output = fieldwrapper(
            self.form['choice_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--choicefield fieldwrapper--select ">
                <div class="select">
                    <select class="" id="id_choice_field" name="choice_field"><option selected="selected" value="1">a</option><option value="2">b</option></select>
                </div>
            </div>
        ''')  # noqa

    def test_fieldwrapper_radio_select(self):
        html_output = fieldwrapper(
            self.form['radio_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--choicefield fieldwrapper--radioselect ">
                <ul id="id_radio_field">
                    <li><label for="id_radio_field_0"><input checked="checked" class="" id="id_radio_field_0" name="radio_field" type="radio" value="1" />a</label></li>
                    <li><label for="id_radio_field_1"><input class="" id="id_radio_field_1" name="radio_field" type="radio" value="2" />b</label></li>
                </ul>
            </div>
        ''')  # noqa

    def test_fieldwrapper_checkbox(self):
        html_output = fieldwrapper(
            self.form['checkbox_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--booleanfield fieldwrapper--checkboxinput ">
                <label class="checkbox control">
                    <input checked="checked" class="" id="id_checkbox_field" name="checkbox_field" type="checkbox" />
                    <span class="control-indicator"></span>
                    <span class="control-label">
                        Checkbox field
                    </span>
                </label>
            </div>
        ''')  # noqa

        html_output = fieldwrapper(
            self.form['checkbox_field'],
            control_label='Different label',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--booleanfield fieldwrapper--checkboxinput ">
                <label class="checkbox control">
                    <input checked="checked" class="" id="id_checkbox_field" name="checkbox_field" type="checkbox" />
                    <span class="control-indicator"></span>
                    <span class="control-label">
                        Different label
                    </span>
                </label>
            </div>
        ''')  # noqa

        html_output = fieldwrapper(
            self.form['checkbox_field'],
            control_label='',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--booleanfield fieldwrapper--checkboxinput ">
                <label class="checkbox control">
                    <input checked="checked" class="" id="id_checkbox_field" name="checkbox_field" type="checkbox" />
                    <span class="control-indicator"></span>
                    <span class="control-label"></span>
                </label>
            </div>
        ''')  # noqa

        html_output = fieldwrapper(
            self.form['checkbox_help_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--booleanfield fieldwrapper--checkboxinput ">
                <label class="checkbox control">
                    <input checked="checked" class="" id="id_checkbox_help_field" name="checkbox_help_field" type="checkbox" />
                    <span class="control-indicator"></span>
                    <span class="control-label">Help Me!</span>
                </label>
            </div>
        ''')  # noqa

        html_output = fieldwrapper(
            self.form['checkbox_help_field'],
            control_label='',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--booleanfield fieldwrapper--checkboxinput ">
                <label class="checkbox control">
                    <input checked="checked" class="" id="id_checkbox_help_field" name="checkbox_help_field" type="checkbox" />
                    <span class="control-indicator"></span>
                    <span class="control-label"></span>
                </label>
                <div class="helptext">Help Me!</div>
            </div>
        ''')  # noqa

    def test_fieldwrapper_password(self):
        html_output = fieldwrapper(
            self.form['pass_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--charfield fieldwrapper--passwordinput fieldwrapper--icon">
                <input class="input-field" id="id_pass_field" name="pass_field" type="password" />
                <button class="input-icon input-icon__password" type="button" tabindex="-1">{}</button>
            </div>
        '''.format(self.eye_svg))  # noqa

    def test_fieldwrapper_date(self):
        html_output = fieldwrapper(
            self.form['date_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--datefield fieldwrapper--dateinput fieldwrapper--icon">
                <input class="input-field" id="id_date_field" name="date_field" type="text" value="1940-10-9" />
                <label class="label--inline-edit" for="id_date_field" title="Edit Date field">{}</label>
            </div>
        '''.format(self.date_range_svg))  # noqa

    def test_fieldwrapper_field_kwargs(self):
        html_output = fieldwrapper(
            self.form['char_field'],
            **{
                'class': ' test class  ',
                'data_some': 'data',
                'placeholder': 'Fill in some test data'
            }
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--charfield fieldwrapper--textinput ">
               <input class="input-field test class" data-some="data" id="id_char_field" name="char_field" type="text" value="abc" placeholder="Fill in some test data" />
            </div>
        ''')  # noqa

    def test_fieldwrapper_before_after_field(self):
        html_output = fieldwrapper(
            self.form['char_field'],
            after_field='@',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--charfield fieldwrapper--textinput ">
                <input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" />@
            </div>
        ''')  # noqa

        html_output = fieldwrapper(
            self.form['char_field'],
            before_field='!',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--charfield fieldwrapper--textinput ">
                !<input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" />
            </div>
        ''')  # noqa

        html_output = fieldwrapper(
            self.form['char_field'],
            before_field='!',
            after_field='@',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--charfield fieldwrapper--textinput ">
                !<input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" />@
            </div>
        ''')  # noqa

        html_output = fieldwrapper(
            self.form['pass_field'],
            before_field='!',
            after_field='@',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--charfield fieldwrapper--passwordinput fieldwrapper--icon">
                !<input class="input-field" id="id_pass_field" name="pass_field" type="password" />
                <button class="input-icon input-icon__password" type="button" tabindex="-1">{}</button>
            </div>
        '''.format(self.eye_svg))  # noqa

    def test_fieldwrapper_unit_text_left_right(self):
        html_output = fieldwrapper(
            self.form['char_field'],
            unit_text_left='$',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--charfield fieldwrapper--textinput fieldwrapper--unit">
               <i class="input-unit input-unit--left">$</i>
               <input class="input-field input-field--unit-left" id="id_char_field" name="char_field" type="text" value="abc" />
            </div>
        ''')  # noqa

        html_output = fieldwrapper(
            self.form['char_field'],
            unit_text_left='$',
            unit_text_right='KM',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--charfield fieldwrapper--textinput fieldwrapper--unit">
               <i class="input-unit input-unit--left">$</i>
               <input class="input-field input-field--unit-left input-field--unit-right" id="id_char_field" name="char_field" type="text" value="abc" />
               <i class="input-unit input-unit--right">KM</i>
            </div>
        ''')  # noqa

        html_output = fieldwrapper(
            self.form['pass_field'],
            unit_text_left='$',
            unit_text_right='KM',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper fieldwrapper--charfield fieldwrapper--passwordinput fieldwrapper--icon">
                <input class="input-field" id="id_pass_field" name="pass_field" type="password" />
                <button class="input-icon input-icon__password" type="button" tabindex="-1">{}</button>
            </div>
        '''.format(self.eye_svg))  # noqa


class FormFieldwrapperCombinedTestCase(
        DummyFormMixin, SvgIconsMixin, SimpleTestCase):
    maxDiff = None

    def test_fieldwrapper_combined(self):
        html_output = fieldwrapper_combined(
            self.form['char_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput ">
                       <input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" placeholder="Char field" />
                    </div>
                </div>
            </div>
        ''')  # noqa

        html_output = fieldwrapper_combined(
            self.form['char_field'],
            self.form['int_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput ">
                       <input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" placeholder="Char field" />
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />
                    </div>
                </div>
            </div>
        ''')  # noqa

    def test_fieldwrapper_combined_class(self):
        html_output = fieldwrapper_combined(
            self.form['char_field'],
            self.form['int_field'],
            fieldwrapper_class='  test class  ',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper test class">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput ">
                       <input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" placeholder="Char field" />
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />
                    </div>
                </div>
            </div>
        ''')  # noqa

    def test_fieldwrapper_combined_errors(self):
        self.form.add_error('char_field', forms.ValidationError('Err'))

        html_output = fieldwrapper_combined(
            self.form['char_field'],
            self.form['int_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput error">
                       <input class="input-field error" id="id_char_field" name="char_field" type="text" value="abc" placeholder="Char field" />
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />
                    </div>
                </div>
                <ul class="errorlist"><li>Err</li></ul>
            </div>
        ''')  # noqa

        self.form.add_error('int_field', forms.ValidationError('Err2'))

        html_output = fieldwrapper_combined(
            self.form['char_field'],
            self.form['int_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput error">
                       <input class="input-field error" id="id_char_field" name="char_field" type="text" value="abc" placeholder="Char field" />
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput error">
                       <input class="input-field error" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />
                    </div>
                </div>
                <ul class="errorlist"><li>Err</li></ul>
                <ul class="errorlist"><li>Err2</li></ul>
            </div>
        ''')  # noqa

        html_output = fieldwrapper_combined(
            self.form['char_field'],
            self.form['int_field'],
            suppress_errors=True,
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput ">
                       <input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" placeholder="Char field" />
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />
                    </div>
                </div>
            </div>
        ''')  # noqa

    def test_fieldwrapper_combined_help_text(self):
        html_output = fieldwrapper_combined(
            self.form['help_field'],
            self.form['int_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput ">
                       <input class="input-field" id="id_help_field" name="help_field" type="text" value="abc" placeholder="Help field" />
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />
                    </div>
                </div>
                <div class="helptext">Help Text</div>
            </div>
        ''')  # noqa

        html_output = fieldwrapper_combined(
            self.form['help_field'],
            self.form['help_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput ">
                       <input class="input-field" id="id_help_field" name="help_field" type="text" value="abc" placeholder="Help field" />
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput ">
                       <input class="input-field" id="id_help_field" name="help_field" type="text" value="abc" placeholder="Help field" />
                    </div>
                </div>
                <div class="helptext">Help Text</div>
                <div class="helptext">Help Text</div>
            </div>
        ''')  # noqa

        self.form.add_error('help_field', forms.ValidationError('Err'))
        self.form.add_error('int_field', forms.ValidationError('Err2'))

        html_output = fieldwrapper_combined(
            self.form['help_field'],
            self.form['int_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput error">
                       <input class="input-field error" id="id_help_field" name="help_field" type="text" value="abc" placeholder="Help field" />
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput error">
                       <input class="input-field error" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />
                    </div>
                </div>
                <div class="helptext">Help Text</div>
                <ul class="errorlist"><li>Err</li></ul>
                <ul class="errorlist"><li>Err2</li></ul>
            </div>
        ''')  # noqa

    def test_fieldwrapper_combined_select(self):
        html_output = fieldwrapper_combined(
            self.form['choice_field'],
            self.form['int_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--choicefield fieldwrapper--select select">
                        <select class="" id="id_choice_field" name="choice_field"><option selected="selected" value="1">a</option><option value="2">b</option></select>
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />
                    </div>
                </div>
            </div>
        ''')  # noqa

    def test_fieldwrapper_combined_password(self):
        html_output = fieldwrapper_combined(
            self.form['pass_field'],
            self.form['int_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--passwordinput fieldwrapper--icon">
                        <input class="input-field" id="id_pass_field" name="pass_field" type="password" placeholder="Pass field" />
                        <button class="input-icon input-icon__password" type="button" tabindex="-1">{}</button>
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />
                    </div>
                </div>
            </div>
        '''.format(self.eye_svg))  # noqa

    def test_fieldwrapper_combined_date(self):
        html_output = fieldwrapper_combined(
            self.form['date_field'],
            self.form['int_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--datefield fieldwrapper--dateinput fieldwrapper--icon">
                        <input class="input-field" id="id_date_field" name="date_field" type="text" value="1940-10-9" placeholder="Date field" />
                        <label class="label--inline-edit" for="id_date_field" title="Edit Date field">{}</label>
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />
                    </div>
                </div>
            </div>
        '''.format(self.date_range_svg))  # noqa

    def test_fieldwrapper_combined_field_kwargs(self):
        html_output = fieldwrapper_combined(
            self.form['char_field'],
            self.form['int_field'],
            field0_class=' test class  ',
            field0_data_some='data',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput ">
                       <input class="input-field test class" data-some="data" id="id_char_field" name="char_field" type="text" value="abc" placeholder="Char field" />
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />
                    </div>
                </div>
            </div>
        ''')  # noqa

        html_output = fieldwrapper_combined(
            self.form['char_field'],
            self.form['int_field'],
            field1_class=' test class  ',
            field1_data_some='data',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput ">
                       <input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" placeholder="Char field" />
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field test class" data-some="data" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />
                    </div>
                </div>
            </div>
        ''')  # noqa

        html_output = fieldwrapper_combined(
            self.form['char_field'],
            self.form['int_field'],
            field0_class=' test class  ',
            field0_data_some='data',
            field1_class='test',
            field1_data_other='{}',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput ">
                       <input class="input-field test class" data-some="data" id="id_char_field" name="char_field" type="text" value="abc" placeholder="Char field" />
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field test" data-other="{}" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />
                    </div>
                </div>
            </div>
        ''')  # noqa

    def test_fieldwrapper_combined_field_placeholder(self):
        html_output = fieldwrapper_combined(
            self.form['char_field'],
            self.form['int_field'],
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput ">
                       <input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" placeholder="Char field" />
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />
                    </div>
                </div>
            </div>
        ''')  # noqa

        html_output = fieldwrapper_combined(
            self.form['char_field'],
            self.form['int_field'],
            field0_placeholder='',
            field1_placeholder='',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput ">
                       <input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" placeholder="" />
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" type="number" value="1" placeholder="" />
                    </div>
                </div>
            </div>
        ''')  # noqa

        html_output = fieldwrapper_combined(
            self.form['char_field'],
            self.form['int_field'],
            field0_placeholder='Test',
            field1_placeholder='Other',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput ">
                       <input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" placeholder="Test" />
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" type="number" value="1" placeholder="Other" />
                    </div>
                </div>
            </div>
        ''')  # noqa

    def test_fieldwrapper_combined_before_after_field(self):
        html_output = fieldwrapper_combined(
            self.form['char_field'],
            self.form['int_field'],
            field0_before_field='!',
            field0_after_field='@',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput ">
                       !<input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" placeholder="Char field" />@
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />
                    </div>
                </div>
            </div>
        ''')  # noqa

        html_output = fieldwrapper_combined(
            self.form['char_field'],
            self.form['int_field'],
            field0_before_field='!',
            field0_after_field='@',
            field1_before_field='#',
            field1_after_field='$',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput ">
                       !<input class="input-field" id="id_char_field" name="char_field" type="text" value="abc" placeholder="Char field" />@
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       #<input class="input-field" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />$
                    </div>
                </div>
            </div>
        ''')  # noqa

        html_output = fieldwrapper_combined(
            self.form['pass_field'],
            self.form['int_field'],
            field0_before_field='!',
            field0_after_field='@',  # ignored
            field1_before_field='#',
            field1_after_field='$',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--passwordinput fieldwrapper--icon">
                        !<input class="input-field" id="id_pass_field" name="pass_field" type="password" placeholder="Pass field" />
                        <button class="input-icon input-icon__password" type="button" tabindex="-1">{}</button>
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       #<input class="input-field" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />$
                    </div>
                </div>
            </div>
        '''.format(self.eye_svg))  # noqa

    def test_fieldwrapper_combined_unit_text_left_right(self):
        html_output = fieldwrapper_combined(
            self.form['char_field'],
            self.form['int_field'],
            field0_unit_text_left='$',
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput fieldwrapper--unit">
                       <i class="input-unit input-unit--left">$</i>
                       <input class="input-field input-field--unit-left" id="id_char_field" name="char_field" type="text" value="abc" placeholder="Char field" />
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />
                    </div>
                </div>
            </div>
        ''')  # noqa

        html_output = fieldwrapper_combined(
            self.form['char_field'],
            self.form['int_field'],
            field0_unit_text_left='$',
            field0_unit_text_right='KM',
            field1_unit_text_left=mark_safe('&euro;'),
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--textinput fieldwrapper--unit">
                       <i class="input-unit input-unit--left">$</i>
                       <input class="input-field input-field--unit-left input-field--unit-right" id="id_char_field" name="char_field" type="text" value="abc" placeholder="Char field" />
                       <i class="input-unit input-unit--right">KM</i>
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput fieldwrapper--unit">
                       <i class="input-unit input-unit--left">&euro;</i>
                       <input class="input-field input-field--unit-left" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />
                    </div>
                </div>
            </div>
        ''')  # noqa

        html_output = fieldwrapper_combined(
            self.form['pass_field'],
            self.form['int_field'],
            field0_unit_text_left='$',
            field0_unit_text_right='KM',
            field1_unit_text_left=mark_safe('&euro;'),
        )
        self.assertHTMLEqual(html_output, '''
            <div class="fieldwrapper ">
                <div class="fieldwrapper-combined">
                    <div class="fieldwrapper-combined__field fieldwrapper--charfield fieldwrapper--passwordinput fieldwrapper--icon">
                        <input class="input-field" id="id_pass_field" name="pass_field" type="password" placeholder="Pass field" />
                        <button class="input-icon input-icon__password" type="button" tabindex="-1">{}</button>
                    </div>
                    <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput fieldwrapper--unit">
                       <i class="input-unit input-unit--left">&euro;</i>
                       <input class="input-field input-field--unit-left" id="id_int_field" name="int_field" type="number" value="1" placeholder="Int field" />
                    </div>
                </div>
            </div>
        '''.format(self.eye_svg))  # noqa

    def test_fieldwrapper_combined_uses_widget_placeholders(self):
        html_output = fieldwrapper_combined(
            self.form['int_field'],
            self.form['int_field'],
            self.form['int_field'],
            field0_placeholder='Eerste',
            field1_placeholder='Tweede',
            field2_placeholder='Derde'
        )

        html = '''
           <div class="fieldwrapper ">
               <div class="fieldwrapper-combined">
                   <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" placeholder="Eerste" type="number" value="1" />
                   </div>
                   <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" placeholder="Tweede" type="number" value="1" />
                   </div>
                   <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" placeholder="Derde" type="number" value="1" />
                   </div>
               </div>
           </div>
        '''  # noqa

        self.assertHTMLEqual(html_output, html)

        self.form.fields['int_field'].widget.attrs.update(
            placeholder='lorem ipsum')

        html_output = fieldwrapper_combined(
            self.form['int_field'],
            self.form['int_field'],
            self.form['int_field'],
            field0_placeholder='Eerste',
            field1_placeholder='Tweede',
            field2_placeholder='Derde'
        )

        self.assertHTMLEqual(html_output, html)

        html_output = fieldwrapper_combined(
            self.form['int_field'],
            self.form['int_field'],
            self.form['int_field'],
        )

        self.assertHTMLEqual(html_output, '''
           <div class="fieldwrapper ">
               <div class="fieldwrapper-combined">
                   <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" placeholder="lorem ipsum" type="number" value="1" />
                   </div>
                   <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" placeholder="lorem ipsum" type="number" value="1" />
                   </div>
                   <div class="fieldwrapper-combined__field fieldwrapper--integerfield fieldwrapper--numberinput ">
                       <input class="input-field" id="id_int_field" name="int_field" placeholder="lorem ipsum" type="number" value="1" />
                   </div>
               </div>
           </div>
        ''')  # noqa




#!/usr/bin/env python

from setuptools import find_packages, setup

version = '1.0'

setup(
    name='django-form-tags',
    packages=find_packages(),
    version=version,
    description='Django templatetags to easily render fieldholders, '
                'fieldwrappers and fields.',
    long_description=open('README.rst').read(),
    author='Sven Groot (Mediamoose)',
    author_email='sven@mediamoose.nl',
    url='https://gitlab.com/mediamoose/django-form-tags/tree/v{}'.format(version),  # noqa
    download_url='https://gitlab.com/mediamoose/django-form-tags/repository/v{}/archive.tar.gz'.format(version),  # noqa
    include_package_data=True,
    install_requires=[
        'django>=1.11',
        'django-svg-templatetag>=1.0.1',
    ],
    license='MIT',
    zip_safe=False,
    keywords=['forms', 'templatetags', 'django'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)
